@@ -7,6 +7,8 @@ from .sqlcliexception import SQLCliException
 import click
 import time
 import os
+import re
+import copy
 from time import strftime, localtime
 from multiprocessing import Lock
 import traceback
@@ -14,25 +16,19 @@ import traceback
 
 class SQLExecute(object):
     conn = None                         # 数据库连接
-    options = None                      # 用户设置的各种选项
     logfile = None                      # 打印的日志文件
     sqlscript = None                    # 需要执行的SQL脚本
     SQLMappingHandler = None            # SQL重写处理
     m_Current_RunningSQL = None         # 目前程序运行的当前SQL
     Console = False                     # 屏幕输出Console
     logger = None                       # 日志输出
-    m_Worker_Name = None                # 为每个SQLExecute实例起一个名字，便于统计分析
 
-    # 进程锁, 用来在输出perf文件的时候控制并发写文件
-    m_PerfFileLocker = Lock()
+    m_PerfFileLocker = Lock()           # # 进程锁, 用来在输出perf文件的时候控制并发写文件
 
-    # 进程终止标志
-    m_Shutdown_Flag = False             # 在程序的每个语句，Sleep间歇中都会判断这个标志
-    m_Abort_Flag = False                # 应用被强行终止，可能会有SQL错误
+    m_Shutdown_Flag = False             # 在程序的每个语句，Sleep间歇中都会判断这个进程终止标志
 
-    # SQLPerf文件
-    SQLPerfFile = None
-    SQLPerfFileHandle = None
+    SQLPerfFile = None                  # SQLPerf文件
+    SQLPerfFileHandle = None            # SQLPerf文件句柄
 
     def __init__(self):
         # 设置一些默认的参数
@@ -42,6 +38,17 @@ class SQLExecute(object):
                         'KAFKA_SERVERS': None,
                         "LOB_LENGTH": 20, 'FLOAT_FORMAT': '%.7g', 'DOUBLE_FORMAT': '%0.10g'}
 
+        # 记录最后SQL返回的结果
+        self.LastAffectedRows = 0
+        self.LastSQLResult = None
+        self.LastElapsedTime = 0
+
+        # 当前Execute的WorkerName
+        self.m_Worker_Name = None
+
+    def set_Worker_Name(self, p_szWorkerName):
+        self.m_Worker_Name = p_szWorkerName
+
     def set_logfile(self, p_logfile):
         self.logfile = p_logfile
 
@@ -102,6 +109,39 @@ class SQLExecute(object):
             start = time.time()
             self.m_Current_RunningSQL = sql
 
+            # 检查SQL中是否包含特殊内容
+            # lastsqlresult.LastAffectedRows
+            if sql.find("%lastsqlresult.LastAffectedRows%") != -1:
+                sql = sql.replace("%lastsqlresult.LastAffectedRows%", str(self.LastAffectedRows))
+                if self.options["ECHO"].upper() == 'ON' and self.logfile is not None:
+                    # SQL已经发生了改变
+                    click.echo(SQLFormatWithPrefix(
+                        "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
+                if p_sqlscript is not None:
+                    click.echo(SQLFormatWithPrefix(
+                        "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
+            # lastsqlresult.LastSQLResult[X][Y]
+            matchObj = re.search(r"%lastsqlresult.LastSQLResult\[(\d+)\]\[(\d+)\]%",
+                                 sql, re.IGNORECASE | re.DOTALL)
+            if matchObj:
+                m_Searched = matchObj.group(0)
+                m_nRow = int(matchObj.group(1))
+                m_nColumn = int(matchObj.group(2))
+                if m_nRow >= len(self.LastSQLResult):
+                    m_Result = ""
+                elif m_nColumn >= len(self.LastSQLResult[m_nRow]):
+                    m_Result = ""
+                else:
+                    m_Result = str(self.LastSQLResult[m_nRow][m_nColumn])
+                sql = sql.replace(m_Searched, m_Result)
+                if self.options["ECHO"].upper() == 'ON' and self.logfile is not None:
+                    # SQL已经发生了改变
+                    click.echo(SQLFormatWithPrefix(
+                        "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
+                if p_sqlscript is not None:
+                    click.echo(SQLFormatWithPrefix(
+                        "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
+
             # 执行SQL
             try:
                 # 首先假设这是一个特殊命令
@@ -113,6 +153,8 @@ class SQLExecute(object):
                     if self.options["WHENEVER_SQLERROR"] == "EXIT":
                         raise SQLCliException("Not Connected. ")
                     else:
+                        self.LastAffectedRows = 0
+                        self.LastSQLResult = None
                         yield None, None, None, "Not connected. "
 
                 # 执行正常的SQL语句
@@ -127,6 +169,9 @@ class SQLExecute(object):
                             (title, result, headers, status, m_FetchStatus, m_FetchedRows) = \
                                 self.get_result(cur, rowcount)
                             rowcount = m_FetchedRows
+
+                            self.LastAffectedRows = rowcount
+                            self.LastSQLResult = copy.copy(result)
                             yield title, result, headers, status
                             if not m_FetchStatus:
                                 break
@@ -165,12 +210,15 @@ class SQLExecute(object):
                                 str(e).find("SQLException") != -1 or
                                 str(e).find("SQLDataException") != -1 or
                                 str(e).find("SQLTransactionRollbackException") != -1 or
+                                str(e).find("SQLTransientConnectionException") != -1 or
                                 str(e).find('time data') != -1
                         ):
                             # 发生了SQL语法错误
                             if self.options["WHENEVER_SQLERROR"] == "EXIT":
                                 raise SQLCliException(str(e))
                             else:
+                                self.LastAffectedRows = 0
+                                self.LastSQLResult = None
                                 yield None, None, None, str(e)
                         else:
                             # 发生了其他不明错误
@@ -185,12 +233,15 @@ class SQLExecute(object):
                     if "SQLCLI_DEBUG" in os.environ:
                         print('traceback.print_exc():\n%s' % traceback.print_exc())
                         print('traceback.format_exc():\n%s' % traceback.format_exc())
+                    self.LastAffectedRows = 0
+                    self.LastSQLResult = None
                     yield None, None, None, str(e.message)
 
             self.m_Current_RunningSQL = None
 
             # 如果需要，打印语句执行时间
             end = time.time()
+            self.LastElapsedTime = end - start
             if self.options['TIMING'].upper() == 'ON':
                 if sql.strip().upper() not in ('EXIT', 'QUIT'):
                     yield None, None, None, 'Running time elapsed: %9.2f Seconds' % (end - start)
@@ -315,25 +366,44 @@ class SQLExecute(object):
             return
 
         # 多进程，多线程写入，考虑锁冲突
-        self.m_PerfFileLocker.acquire()
-        if not os.path.exists(self.SQLPerfFile):
-            # 如果文件不存在，创建文件，并写入文件头信息
-            self.SQLPerfFileHandle = open(self.SQLPerfFile, "a", encoding="utf-8")
-            self.SQLPerfFileHandle.write("Script\tStarted\telapsed\tSQLPrefix\tSQLStatus\tErrorMessage\tthread_name\n")
-            self.SQLPerfFileHandle.close()
+        try:
+            self.m_PerfFileLocker.acquire()
+            if not os.path.exists(self.SQLPerfFile):
+                # 如果文件不存在，创建文件，并写入文件头信息
+                self.SQLPerfFileHandle = open(self.SQLPerfFile, "a", encoding="utf-8")
+                self.SQLPerfFileHandle.write("Script\tStarted\telapsed\tSQLPrefix\t"
+                                             "SQLStatus\tErrorMessage\tthread_name\t"
+                                             "Copies\tFinished\n")
+                self.SQLPerfFileHandle.close()
 
-        # 打开Perf文件
-        self.SQLPerfFileHandle = open(self.SQLPerfFile, "a", encoding="utf-8")
-        # 写入内容信息
-        self.SQLPerfFileHandle.write(
-            "'" + str(os.path.basename(self.sqlscript)) + "'\t" +
-            "'" + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(p_SQLResult["StartedTime"])) + "'\t" +
-            "%8.2f" % p_SQLResult["elapsed"] + "\t" +
-            "'" + str(p_SQLResult["SQL"][0:40]).replace("\n", " ").replace("\t", "    ") + "'\t" +
-            str(p_SQLResult["SQLStatus"]) + "\t" +
-            "'" + str(p_SQLResult["ErrorMessage"]).replace("\n", " ").replace("\t", "    ") + "'\t" +
-            "'" + str(p_SQLResult["thread_name"] + "'" + "\n")
-        )
-        self.SQLPerfFileHandle.flush()
-        self.SQLPerfFileHandle.close()
-        self.m_PerfFileLocker.release()
+            if len(str(p_SQLResult["thread_name"]).split('-')) == 3:
+                # 对于多线程运行，这里的thread_name通常为JOB_NAME, 副本数，循环次数
+                m_ThreadName = str(p_SQLResult["thread_name"]).split('-')[0]
+                m_Copies = str(p_SQLResult["thread_name"]).split('-')[1]
+                m_CurrentLoop = int(str(p_SQLResult["thread_name"]).split('-')[2]) + 1
+            else:
+                # 对于单线程运行，这里的thread_name通常为JOB_NAME, 1， 1
+                m_ThreadName = str(p_SQLResult["thread_name"])
+                m_Copies = 1
+                m_CurrentLoop = 1
+
+            # 打开Perf文件
+            self.SQLPerfFileHandle = open(self.SQLPerfFile, "a", encoding="utf-8")
+            # 写入内容信息
+            self.SQLPerfFileHandle.write(
+                "'" + str(os.path.basename(self.sqlscript)) + "'\t" +
+                "'" + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(p_SQLResult["StartedTime"])) + "'\t" +
+                "%8.2f" % p_SQLResult["elapsed"] + "\t" +
+                "'" + str(p_SQLResult["SQL"]).replace("\n", " ").replace("\t", "    ") + "'\t" +
+                str(p_SQLResult["SQLStatus"]) + "\t" +
+                "'" + str(p_SQLResult["ErrorMessage"]).replace("\n", " ").replace("\t", "    ") + "'\t" +
+                "'" + str(m_ThreadName) + "'\t" +
+                str(m_Copies) + "'\t" + str(m_CurrentLoop) +
+                "\n"
+            )
+            self.SQLPerfFileHandle.flush()
+            self.SQLPerfFileHandle.close()
+        except Exception as ex:
+            print("Internal error:: perf file write not complete. " + repr(ex))
+        finally:
+            self.m_PerfFileLocker.release()
